@@ -1,12 +1,13 @@
 lapply(list.files(system.file("testthat", package = "mlr3"), pattern = "^helper.*\\.[rR]$", full.names = TRUE), source)
 
+#FIXME: This function should be exported so it can be used for tests in other packages
 expect_tuner = function(tuner) {
   expect_r6(tuner, "Tuner",
-    public = c("tune", "param_set"),
-    private = "tune_internal"
+    public = c("optimize", "param_set"),
+    private = ".optimize"
   )
   expect_is(tuner$param_set, "ParamSet")
-  expect_function(tuner$tune, args = "instance")
+  expect_function(tuner$optimize, args = "inst")
 }
 
 expect_terminator = function(term) {
@@ -16,6 +17,7 @@ expect_terminator = function(term) {
   expect_is(term$param_set, "ParamSet")
 }
 
+#FIXME: This function should be exported so it can be used for tests in other packages
 # test an implemented subclass tuner by running a couple of standard tests
 # on a simple example
 # term_evals: how we configure the Terminator
@@ -32,51 +34,49 @@ test_tuner = function(key, ..., n_dim = 1L, term_evals = 2L, real_evals = term_e
       ParamInt$new("minsplit", lower = 1, upper = 9)
     ))
   }
-  term = term("evals", n_evals = term_evals)
-  inst = TuningInstance$new(tsk("iris"), lrn("classif.rpart"), rsmp("holdout"), msr("classif.ce"), ps, term)
+  term = trm("evals", n_evals = term_evals)
+  inst = TuningInstanceSingleCrit$new(tsk("iris"), lrn("classif.rpart"), rsmp("holdout"), msr("classif.ce"), ps, term)
   tuner = tnr(key, ...)
   expect_tuner(tuner)
-  tuner$tune(inst)
-  bmr = inst$bmr
+  tuner$optimize(inst)
+  archive = inst$archive
 
-  expect_data_table(bmr$data, nrows = real_evals)
-  expect_equal(inst$n_evals, real_evals)
+  expect_data_table(archive$data(), nrows = real_evals)
+  expect_equal(inst$archive$n_evals, real_evals)
 
-  r = inst$result
-  sc = r$tune_x
-  sp = r$perf
-  expect_list(sc, len = n_dim)
+  x_opt = inst$result_x_domain
+  y_opt = inst$result_y
+  expect_list(x_opt, len = n_dim)
   if (n_dim == 1)
-    expect_named(sc, c("cp"))
+    expect_named(x_opt, c("cp"))
   else
-    expect_named(sc, c("cp", "minsplit"))
-  expect_numeric(sp, len = 1L)
-  expect_named(sp, "classif.ce")
+    expect_named(x_opt, c("cp", "minsplit"))
+  expect_numeric(y_opt, len = 1L)
+  expect_named(y_opt, "classif.ce")
   list(tuner = tuner, inst = inst)
 }
 
 # test an implemented subclass tuner by running a test with dependent params
 # returns: tune_result and instance
 test_tuner_dependencies = function(key, ..., term_evals = 2L) {
-  term = term("evals", n_evals = term_evals)
+  term = trm("evals", n_evals = term_evals)
   ll = LearnerRegrDepParams$new()
-  inst = TuningInstance$new(tsk("boston_housing"), ll, rsmp("holdout"), msr("regr.mse"), ll$param_set, term)
+  inst = TuningInstanceSingleCrit$new(tsk("boston_housing"), ll, rsmp("holdout"), msr("regr.mse"), ll$param_set, term)
   tuner = tnr(key, ...)
   expect_tuner(tuner)
-  tuner$tune(inst)
-  bmr = inst$bmr
+  tuner$optimize(inst)
+  archive = inst$archive
 
-  expect_data_table(bmr$data, nrows = term_evals)
-  expect_equal(inst$n_evals, term_evals)
+  expect_data_table(archive$data(), nrows = term_evals)
+  expect_equal(inst$archive$n_evals, term_evals)
 
-  r = inst$result
-  sc = r$tune_x
-  sp = r$perf
-  expect_list(sc)
-  expect_names(names(sc), subset.of = c("xx", "yy", "cp"))
-  expect_numeric(sp, len = 1L)
-  expect_numeric(sp, len = 1L)
-  expect_names(names(sp), identical.to = "regr.mse")
+  x_opt = inst$result_x_domain
+  y_opt = inst$result_y
+  expect_list(x_opt)
+  expect_names(names(x_opt), subset.of = c("xx", "yy", "cp"))
+  expect_numeric(y_opt, len = 1L)
+  expect_numeric(y_opt, len = 1L)
+  expect_names(names(y_opt), identical.to = "regr.mse")
   list(tuner = tuner, inst = inst)
 }
 
@@ -94,15 +94,33 @@ TEST_MAKE_PS1 = function(n_dim = 1L) {
     ))
   }
 }
-TEST_MAKE_INST1 = function(values = NULL, folds = 2L, measures = msr("classif.ce"), n_dim = 1L, term_evals = 5L) {
+TEST_MAKE_INST1 = function(values = NULL, folds = 2L,
+  measure = msr("classif.ce"), n_dim = 1L, term_evals = 5L,
+  store_resample_results = TRUE) {
   ps = TEST_MAKE_PS1(n_dim = n_dim)
   lrn = mlr_learners$get("classif.rpart")
   if (!is.null(values)) {
     lrn$param_set$values = values
   }
   rs = rsmp("cv", folds = folds)
-  term = term("evals", n_evals = term_evals)
-  inst = TuningInstance$new(tsk("iris"), lrn, rs, measures, ps, term)
+  term = trm("evals", n_evals = term_evals)
+  inst = TuningInstanceSingleCrit$new(tsk("iris"), lrn, rs, measure, ps, term,
+    store_resample_results = store_resample_results)
+  return(inst)
+}
+
+TEST_MAKE_INST1_2D = function(values = NULL, folds = 2L,
+  measures = msrs(c("classif.ce", "classif.acc")),  n_dim = 1L, term_evals = 5L,
+  store_resample_results = TRUE) {
+  ps = TEST_MAKE_PS1(n_dim = n_dim)
+  lrn = mlr_learners$get("classif.rpart")
+  if (!is.null(values)) {
+    lrn$param_set$values = values
+  }
+  rs = rsmp("cv", folds = folds)
+  term = trm("evals", n_evals = term_evals)
+  inst = TuningInstanceMultiCrit$new(tsk("iris"), lrn, rs, measures, ps, term,
+                                      store_resample_results = store_resample_results)
   return(inst)
 }
 
@@ -118,18 +136,18 @@ TEST_MAKE_PS2 = function() {
   ps$add_dep("yy", on = "xx", cond = CondEqual$new("a"))
   return(ps)
 }
-TEST_MAKE_INST2 = function(measures = msr("dummy.cp.regr"), term_evals = 5L) {
+TEST_MAKE_INST2 = function(measure = msr("dummy.cp.regr"), term_evals = 5L) {
   ps = TEST_MAKE_PS2()
   ll = LearnerRegrDepParams$new()
   rs = rsmp("holdout")
-  term = term("evals", n_evals = term_evals)
-  inst = TuningInstance$new(tsk("boston_housing"), ll, rs, measures, ps, term)
+  term = trm("evals", n_evals = term_evals)
+  inst = TuningInstanceSingleCrit$new(tsk("boston_housing"), ll, rs, measure, ps, term)
   return(inst)
 }
 
 # a dummy measure which simply returns the cp value of rpart
 # this allows us to 'fake' performance values in unit tests during tuning
-make_dummy_cp_measure = function(type) {
+make_dummy_cp_measure = function(type, minimize = TRUE) {
   if (type == "classif") {
     id = "dummy.cp.classif"
     inh = MeasureClassif
@@ -150,14 +168,16 @@ make_dummy_cp_measure = function(type) {
         super$initialize(
           id = id,
           range = c(0, Inf),
-          minimize = TRUE,
+          minimize = minimize,
           properties = "requires_learner"
         )
         self$fun = fun # allow a fun to transform cp to score
-      },
+      }
+    ),
 
-      score_internal = function(prediction, learner, ...) {
-        self$fun(learner$param_set$values$cp)
+    private = list(
+      .score = function(prediction, learner, ...) {
+        self$fun(learner$param_set$values)
       }
     )
   )
@@ -166,6 +186,8 @@ MeasureDummyCPClassif = make_dummy_cp_measure("classif")
 mlr3::mlr_measures$add("dummy.cp.classif", MeasureDummyCPClassif)
 MeasureDummyCPRegr = make_dummy_cp_measure("regr")
 mlr3::mlr_measures$add("dummy.cp.regr", MeasureDummyCPRegr)
+MeasureDummyCPMaximizeClassif = make_dummy_cp_measure("classif", minimize = FALSE)
+mlr3::mlr_measures$add("dummy.cp.maximize.classif", MeasureDummyCPMaximizeClassif)
 
 LearnerRegrDepParams = R6Class("LearnerRegrDepParams", inherit = LearnerRegr,
   public = list(
@@ -178,14 +200,16 @@ LearnerRegrDepParams = R6Class("LearnerRegrDepParams", inherit = LearnerRegr,
         param_set = param_set,
         properties = c("missings")
       )
-    },
+    }
+  ),
 
-    train_internal = function(task) {
+  private = list(
+    .train = function(task) {
       tn = task$target_names
       return(list())
     },
 
-    predict_internal = function(task) {
+    .predict = function(task) {
       n = task$nrow
       response = rep(99, n)
       PredictionRegr$new(task, response = response)
@@ -193,4 +217,14 @@ LearnerRegrDepParams = R6Class("LearnerRegrDepParams", inherit = LearnerRegr,
   )
 )
 
+MAKE_GL = function() {
+  g = mlr3pipelines::Graph$new()
+  op_ds = mlr3pipelines::PipeOpSubsample$new()
+  op_lrn = mlr3pipelines::PipeOpLearner$new(lrn("classif.rpart"))
+  g$add_pipeop(op_ds)
+  g$add_pipeop(op_lrn)
+  g$add_edge("subsample", "classif.rpart")
+  mlr3pipelines::GraphLearner$new(g)
+}
+
 # mlr3::mlr_learners$add("regr.depparams", LearnerRegrDepParams)
