@@ -1,10 +1,10 @@
 context("AutoTuner")
 
 test_that("AutoTuner / train+predict", {
-  te = term("evals", n_evals = 3)
+  te = trm("evals", n_evals = 4)
   task = tsk("iris")
   ps = TEST_MAKE_PS1(n_dim = 1)
-  ms = MeasureDummyCPClassif$new(fun = function(cp) if (cp == 0.2) 0 else 1) # lets fake a measure, so we control the best config
+  ms = MeasureDummyCPClassif$new(fun = function(pv) if (pv$cp == 0.2) 0 else 1) # lets fake a measure, so we control the best config
   tuner = tnr("grid_search", resolution = 3)
   at = AutoTuner$new(lrn("classif.rpart"), rsmp("holdout"), ms, ps, te, tuner = tuner)
   expect_learner(at)
@@ -12,11 +12,11 @@ test_that("AutoTuner / train+predict", {
   expect_learner(at)
   expect_equal(at$learner$param_set$values, list(xval = 0, cp = 0.2))
   inst = at$tuning_instance
-  expect_benchmark_result(inst$bmr)
-  a = at$archive()
+  a = at$archive$data()
   expect_data_table(a, nrows = 3L)
   r = at$tuning_result
-  expect_equal(r$tune_x, list(cp = 0.2))
+  expect_equal(r$x_domain[[1]], list(cp = 0.2))
+  expect_equal(r$learner_param_vals[[1]], list(xval = 0, cp = 0.2))
   prd = at$predict(task)
   expect_prediction(prd)
   expect_is(at$learner$model, "rpart")
@@ -27,12 +27,12 @@ test_that("AutoTuner / resample", {
   inner_folds = 1L
   inner_evals = 3L
 
-  ms = MeasureDummyCPClassif$new(fun = function(cp) if (cp == 0.2) 0 else 1) # lets fake a measure, so we control the best config
+  ms = MeasureDummyCPClassif$new(fun = function(pv) if (pv$cp == 0.2) 0 else 1) # lets fake a measure, so we control the best config
   tuner = tnr("grid_search", resolution = 3)
   r_inner = rsmp("holdout")
   r_outer = rsmp("cv", folds = 2)
   param_set = TEST_MAKE_PS1()
-  te = term("evals", n_evals = inner_evals)
+  te = trm("evals", n_evals = inner_evals)
   tuner = tnr("grid_search", resolution = 3)
   at = AutoTuner$new(lrn("classif.rpart", predict_type = "prob"), r_inner, ms, param_set, te, tuner)
 
@@ -47,24 +47,22 @@ test_that("AutoTuner / resample", {
     assert_r6(ll, "AutoTuner")
     expect_equal(ll$learner$param_set$values, list(xval = 0, cp = 0.2))
     inst = ll$tuning_instance
-    assert_r6(inst, "TuningInstance")
-    r = inst$result
-    expect_data_table(inst$bmr$data, nrows = inner_evals * inner_folds)
-    expect_data_table(inst$archive(), nrows = inner_evals)
-    expect_numeric(r$perf, len = 1L)
+    assert_r6(inst, "TuningInstanceSingleCrit")
+    expect_data_table(inst$archive$data(), nrows = inner_evals)
+    expect_numeric(inst$result_y, len = 1L)
   })
 })
 
 # we had an issue that the AutoTuner did not return statically configured param in its result
 # see issue #51
 test_that("AutoTuner / param_set", {
-  measures = msr("classif.ce")
-  te = term("evals", n_evals = 3)
+  measure = msr("classif.ce")
+  te = trm("evals", n_evals = 3)
   task = tsk("iris")
   ps = TEST_MAKE_PS1()
   tuner = TunerRandomSearch$new()
   learner = lrn("classif.rpart", cp = 1, maxdepth = 1)
-  at = AutoTuner$new(learner, rsmp("holdout"), measures, ps, te, tuner)
+  at = AutoTuner$new(learner, rsmp("holdout"), measure, ps, te, tuner)
   expect_equal(at$param_set$values[names(at$learner$param_set$values)], at$learner$param_set$values)
   at$train(task)
 
@@ -86,13 +84,13 @@ test_that("AutoTuner / param_set", {
 
 
 test_that("Custom resampling is not allowed", {
-  measures = msr("classif.ce")
-  te = term("evals", n_evals = 3)
+  measure = msr("classif.ce")
+  te = trm("evals", n_evals = 4)
   task = tsk("iris")
   ps = TEST_MAKE_PS1()
   tuner = TunerRandomSearch$new()
   r = rsmp("holdout")$instantiate(task)
-  expect_error(AutoTuner$new(lrn("classif.rpart"), r, measures, ps, te, tuner), "instantiated")
+  expect_error(AutoTuner$new(lrn("classif.rpart"), r, measure, ps, te, tuner), "instantiated")
 })
 
 
@@ -110,9 +108,9 @@ test_that("nested resamppling results are consistent ", {
   lrn = AutoTuner$new(
     learner = lrn("classif.rpart"),
     resampling = rsmp("holdout"),
-    tune_ps = ps,
-    measures = msr("classif.ce"),
-    terminator = term("evals", n_evals = 3),
+    search_space = ps,
+    measure = msr("classif.ce"),
+    terminator = trm("evals", n_evals = 4),
     tuner = tnr("random_search")
   )
 
@@ -122,17 +120,94 @@ test_that("nested resamppling results are consistent ", {
   ll2 = rr$learners[[2]]
   tr1 = ll1$tuning_result
   tr2 = ll2$tuning_result
-  expect_equal(tr1$tune_x, ll1$model$learner$model$control[c("cp", "minsplit")])
-  expect_equal(tr2$tune_x, ll2$model$learner$model$control[c("cp", "minsplit")])
+  expect_equal(tr1$x_domain[[1]], ll1$model$learner$model$control[c("cp", "minsplit")])
+  expect_equal(tr2$x_domain[[1]], ll2$model$learner$model$control[c("cp", "minsplit")])
 })
 
 test_that("AT training does not change learner in instance args", {
   # we had a bad pointer bug due to missing cloning here
-  #https://github.com/mlr-org/mlr3/issues/428
+  # https://github.com/mlr-org/mlr3/issues/428
   task = tsk("iris")
   ps = TEST_MAKE_PS1()
-  at = AutoTuner$new(lrn("classif.rpart"), rsmp("holdout"), msr("classif.ce"), ps, term("evals", n_evals = 3), TunerRandomSearch$new())
+  at = AutoTuner$new(lrn("classif.rpart"), rsmp("holdout"), msr("classif.ce"), ps, trm("evals", n_evals = 3), TunerRandomSearch$new())
   expect_equal(at$instance_args$learner$param_set$values, list(xval = 0))
   at$train(task)
   expect_equal(at$instance_args$learner$param_set$values, list(xval = 0))
 })
+
+test_that("AutoTuner works with graphlearner", {
+  skip_if_not_installed("mlr3pipelines")
+  requireNamespace("mlr3pipelines")
+
+  gl = MAKE_GL()
+  task = tsk("iris")
+  ms = MeasureDummyCPClassif$new(fun = function(pv) if (pv$classif.rpart.cp == 0.2) 0 else 1)
+  te = trm("evals", n_evals = 4)
+  ps = ParamSet$new(list(
+    ParamDbl$new("classif.rpart.cp", lower = 0.1, upper = 0.3)
+  ))
+  tuner = tnr("grid_search", resolution = 3)
+  at = AutoTuner$new(
+    learner = gl,
+    resampling = rsmp("holdout"),
+    measure = ms,
+    search_space = ps,
+    terminator = te,
+    tuner = tuner)
+
+  expect_learner(at)
+  at$train(task)
+  expect_learner(at)
+  expect_equal(at$learner$param_set$values$classif.rpart.xval, 0)
+  expect_equal(at$learner$param_set$values$classif.rpart.cp, 0.2)
+  inst = at$tuning_instance
+  a = at$archive$data()
+  expect_data_table(a, nrows = 3L)
+  r = at$tuning_result
+  expect_equal(r$x_domain[[1]], list(classif.rpart.cp = 0.2))
+  expect_equal(r$learner_param_vals[[1]]$classif.rpart.xval, 0)
+  expect_equal(r$learner_param_vals[[1]]$classif.rpart.cp, 0.2)
+  prd = at$predict(task)
+  expect_prediction(prd)
+  expect_is(at$learner$model$classif.rpart$model, "rpart")
+})
+
+test_that("Nested resampling works with graphlearner", {
+  skip_if_not_installed("mlr3pipelines")
+  requireNamespace("mlr3pipelines")
+
+  gl = MAKE_GL()
+  task = tsk("iris")
+  ms = MeasureDummyCPClassif$new(fun = function(pv) if (pv$classif.rpart.cp == 0.2) 0 else 1)
+  te = trm("evals", n_evals = 4)
+  ps = ParamSet$new(list(
+    ParamDbl$new("classif.rpart.cp", lower = 0.1, upper = 0.3)
+  ))
+  tuner = tnr("grid_search", resolution = 3)
+  at = AutoTuner$new(
+    learner = gl,
+    resampling = rsmp("holdout"),
+    measure = ms,
+    search_space = ps,
+    terminator = te,
+    tuner = tuner)
+  at$store_tuning_instance = TRUE
+
+  resampling_outer = rsmp("cv", folds = 2)
+  rr = resample(task, at, resampling_outer, store_models = TRUE)
+
+  expect_learner(rr$data$learner[[1]])
+  expect_learner(rr$data$learner[[2]])
+
+  expect_equal(rr$data$learner[[1]]$tuning_result$classif.rpart.cp, 0.2)
+  expect_equal(rr$data$learner[[2]]$tuning_result$classif.rpart.cp, 0.2)
+
+  expect_equal(rr$data$learner[[1]]$learner$param_set$values$classif.rpart.cp, 0.2)
+  expect_equal(rr$data$learner[[2]]$learner$param_set$values$classif.rpart.cp, 0.2)
+
+  expect_data_table(rr$data$learner[[1]]$archive$data(), nrows = 3L)
+  expect_data_table(rr$data$learner[[2]]$archive$data(), nrows = 3L)
+
+  expect_is(rr$data$learner[[1]]$model$learner$model$classif.rpart$model, "rpart")
+  expect_is(rr$data$learner[[1]]$model$learner$model$classif.rpart$model, "rpart")
+})
